import { Rebuilder } from './rebuild';
declare type PackageJSONValue = string | Record<string, unknown>;
export declare enum BuildType {
    Debug = "Debug",
    Release = "Release"
}
export declare class ModuleRebuilder {
    private modulePath;
    private packageJSON;
    private rebuilder;
    constructor(rebuilder: Rebuilder, modulePath: string);
    get buildType(): BuildType;
    get metaPath(): string;
    get metaData(): string;
    get moduleName(): string;
    alreadyBuiltByRebuild(): Promise<boolean>;
    buildNodeGypArgs(prefixedArgs: string[]): Promise<string[]>;
    getSupportedNapiVersions(): Promise<number[] | undefined>;
    getPrebuildInstallRuntimeArgs(): Promise<string[]>;
    getElectronNodeAPIVersion(): number;
    getNapiVersion(): Promise<number | undefined>;
    buildNodeGypArgsFromBinaryField(): Promise<string[]>;
    cacheModuleState(cacheKey: string): Promise<void>;
    isPrebuildInstallNativeModule(): Promise<boolean>;
    packageJSONFieldWithDefault(key: string, defaultValue: PackageJSONValue): Promise<PackageJSONValue>;
    packageJSONField(key: string): Promise<PackageJSONValue | undefined>;
    /**
     * Whether a prebuild-install-based native module exists.
     */
    prebuildInstallNativeModuleExists(): Promise<boolean>;
    private restoreEnv;
    /**
     * If the native module uses prebuildify, check to see if it comes with a prebuilt module for
     * the given platform and arch.
     */
    findPrebuildifyModule(cacheKey: string): Promise<boolean>;
    rebuildNodeGypModule(cacheKey: string): Promise<void>;
    rebuildPrebuildModule(cacheKey: string): Promise<boolean>;
    replaceExistingNativeModule(): Promise<void>;
    runPrebuildInstall(prebuildInstallPath: string): Promise<void>;
    writeMetadata(): Promise<void>;
}
export {};
